;;;; This is a tool for generation of calendars, which can be shown in an Internet Browser. 
;;;; This tool relies on the very old HTML ad hoc stuff, and as such it is obsolete by now.
;;;; Please use <a href = "../xml-in-laml/man/calendar.html"> tools/calendar/xml-in-laml/calendar.scm</a> instead.<p>
;;;; 
;;;; The main function is calendar.
;;;; Besides this function, it is important to understand the input format of appointments in the calendar.
;;;; Also, there are a number of constants, which affect the appearance of the calendar. <p>
;;;;
;;;; This tools requires input (appointments). The tool assumes that the calendar input is the Scheme
;;;; variabel calendar-events. In other words, you must define this variable.
;;;; Below, we explain the format of calendar-events.<p>
;;;;
;;;; This tools requires the libraries general, cgi, html, encode-decode, hex, and time. 
;;;; They are not loaded by the tool itself.

; The LAML library and programs are written by Kurt Normark, Aalborg University, Denmark.
; Copyright (C) 1999  Kurt Normark, normark@cs.auc.dk.
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


; LOG:
; 21.12.98:
; Changed such that multiple appointments on a single day get their own colors.
; boil-together has been changed to return color-frames instead of a simple text.
; The color-function in month-column has been changed to use background colors now that each appointment has its color-frame

; 5.2.98: We introduced various optimalizations in the html output, including the optional one controlled by the
; variable use-style-tag-in-table.

; 5.2.98: We introduced a new week separation which looks real good in Internet Explorer 4.0, but not as good in Netscape 4.01.


;;; About the format of calendar input. 
;;; The calendar input must be present in a variable named calendar-events.
;;; calendar-events must be defined externally to the calendar tool.
;;; Calendar-events is a list of appointments.
;;; A single appointment is, in turn, a list. Thus calendar-events is a list of lists.<p>
;;; 
;;; The detailed format of an appointment is the following: <br>
;;; 
;;; <pre>   (start-time end-time brief-description long-description color url) </pre>
;;; 
;;; Start-time and end-time are integers (a number of seconds since january 1, 1970).
;;; The function time-encoding from the <a href="../../../lib/man/time.html">time library</a> is useful for construction of these
;;; numbers.<p>
;;; 
;;; Brief-description is a string, the contents of which goes into the calendar.
;;; Long-description is a longer description of the event. Currently, this information is not used. <p>
;;; 
;;; Color is a RGB color list (a list of three integers). You can use the color constans of the html Scheme library,
;;; if you want to. <p>
;;; 
;;; Finally, url is the url, to which we link from the calendar entry. If it is the empty string, no link will be made.


(define use-style-tag-in-table #f) ; style tags works in Internet Explorer 4.0, but not in Netscape 4.01

; ---------------------------------------------------------------------------------------------------------------

;;; Event selection functions. 
;;; The event selector functions define accessors into a calendar appointment.

;; Return the start time of a calendar appointment.
(define (event-start-time e) (list-ref e 0))

;; Return the end time of a calendar appointment.
(define (event-end-time e) (list-ref e 1))

;; Return the brief textual description of a calendar appointment.
(define (event-brief-text e) (list-ref e 2))

;; Return the long textual description of a calendar appointment.
(define (event-long-text e) (list-ref e 3))

;; Return the  color of a calendar appointment.
(define (event-color e) (list-ref e 4))

;; Return the url of a calendar appointment.
(define (event-url e) (list-ref e 5))



(define (event-interval e) (list (event-start-time e) (event-end-time e)))


(define cal-title "Prve kalender")


; (define calendar-events ...)

; _____________________________________________________________________________

;;; Calendar parameters. 
;;; In this section we find a number of variables which control the calendar appearance.

;; The calendar font size. Default value is 1.
(define calendar-font-size 1)

;; Week separators: Whether thin og thick (symbols).
(define week-separator-of-calendar 'thin) ; alternative: 'thick

(define (week-sep) 
  (cond ((eq? week-separator-of-calendar 'thin) "")
        ((eq? week-separator-of-calendar 'thick) (hr))
        (else (error "Calendar: week separator problems"))))

;; The width of a month. Default value is 150.
(define month-width 150)

;; The background color of the calendar. A RGB list. Default is white.
(define calendar-background-color white)

; is there an overlap between the intervals, each of which is a list (x y), x <= y.
; function of general interest
(define (interval-overlap? interval-1 interval-2)
  (let ((i1-start (car interval-1))
        (i1-end (cadr interval-1))
        (i2-start (car interval-2))
        (i2-end (cadr interval-2)))
   (or (and (<= i2-start i1-end) (<= i1-end i2-end))
       (and (<= i1-start i2-end) (<= i2-end i1-end))
       ; embedding:
       (and (<= i2-start i1-start) (<= i1-end i2-end))
       (and (<= i1-start i2-start) (<= i2-end i1-end)))))

; Return an interval of first half day.  From 00:00 to 12:00
(define (first-half-day-interval year month day)
 (let ((start (second-count year month day 0 0 0)))
    (list start (- (+ start (quotient seconds-in-a-day 2)) 1))))

; Return an interval of second half day. From 12:00 to 24:00
(define (second-half-day-interval year month day)

 (let ((start (second-count year month day 0 0 0)))
    (list (+ start (quotient seconds-in-a-day 2)) (- (+ start seconds-in-a-day) 1))))

; Return a list of the form (y . m). There are number-of-months entries in the list:
(define (month-list from-month year number-of-months)
 (let ((y-m-l (year-month-list year from-month number-of-months)))
  (map (lambda (y-m) (con (get-month-name (cdr y-m)) " " (as-string (car y-m))))
    y-m-l)))

; enumerate the second counts of month in year:
(define (make-second-counts month year)
  (map (lambda (day)
          (second-count year month day 0 0 0))  ; !!
    (number-interval 1 (days-in-month month year))))

(define (include-style-tag)
  (if use-style-tag-in-table "style=\"font-size: 7pt\" " ""))

; a specialized table function for the calendar:
(define (cal-table border cell-width-list cell-color-function weekend? cell-font-size list-of-list)
  (let ((bdr (string-append "border = " (as-string border)))
       )
    (string-append
     "<table " (include-style-tag) bdr ">"
     (apply string-append
            (map
             (lambda (row)
               (string-append
                "<tr>"
                (apply string-append
                       (map (lambda (cell width color-1)
                              (string-append "<td width = " (as-string width) " " 
                                             "bgcolor = " (string-it (rgb-string-list color-1)) ">"
                                             (con 
                                                  (present-cell cell cell-font-size)
                                                  (if (weekend? row) (hr) ""))                      
                                     "</td>"))
                             row cell-width-list (cell-color-function row)))
                "</tr>"))
             list-of-list))
    "</table>")))

(define (present-cell cell size)
  (if use-style-tag-in-table (as-string cell)  (font-size size (as-string cell))))

; return all elements of list which satiesfy the predicate p
; general:
(define (find-all p lst)
  (find-all-help p lst '()))

(define (find-all-help p lst res)
  (cond ((null? lst) (reverse res))
        ((p (car lst)) (find-all-help p (cdr lst) (cons (car lst) res)))
        (else (find-all-help p (cdr lst) res))))


; Return the text of a day in a calendar, given year, month, day and day-part. 
; Searches in calendar-events.
; part-of-day is either 1 (first half day) or 2 (second half day)
(define (tasks year month day part-of-day)
 (let* ((cal-interval (cond ((= 1 part-of-day) (first-half-day-interval year month day))
                            ((= 2 part-of-day) (second-half-day-interval year month day))
                            (else (error "task half day problem"))))
        (matching-events 
          (find-all
             (lambda (e) (interval-overlap? cal-interval (event-interval e)))
             calendar-events))
        (wnc (week-number-contribution year month day part-of-day)))
  (string-append (if (equal? "" wnc) "" (i wnc)) (boil-together matching-events ""))))


(define (week-number-contribution year month day part-of-day)
  (if (= 1 part-of-day)
      (let* ((sc (second-count year month day 0 0 0))
             (day-number (weekday-number sc)))
        (if (= 1 day-number) ; monday
            (let ((week-num (danish-week-number sc)))
              (as-string week-num))
            ""))
      ""))


; aggreates the matching event to a single string, use separator sep between entries.
(define (boil-together matching-events sep)
  (if (null? matching-events)
      (space 1)
      (let ((res  (apply string-append
                         (map (lambda (s)
                                (string-append s sep))
                              (map (lambda (e) (if  (> (string-length (event-url e)) 0) 
                                                    (color-frame  (font-size calendar-font-size (a-tag (event-url e) (event-brief-text e))) (event-color e))
                                                    (color-frame  (font-size calendar-font-size (event-brief-text e)) (event-color e)) ))
                                    matching-events)))))
        (substring res 0 (- (string-length res) (string-length sep))))))




; Given a particular calendar interval, cal-interval, which is day-number (redundant),
; return the color asked for in the event-list
(define (the-event-color cal-interval day-number)
  (let ((matching-events 
          (find-all
             (lambda (e) (interval-overlap? cal-interval (event-interval e)))
             calendar-events)))
    (if (null? matching-events)
        (if (even? day-number) green1 green2)
        (event-color (car matching-events)))))  


; return a html table for month in year.
; if the last parameter is true, emphasize today in red.
(define (month-column month year today-info)
 (let ((days (days-in-month month year))
       (half-column-width (quotient (- month-width 40) 2)))
  (cal-table 0 
    (list 20 20 half-column-width half-column-width)
    
    ; color function:
    (lambda (row)
     (if (equal? (car row) (week-sep))
      (make-list 4 (if (eq? week-separator-of-calendar 'thin) black grey1))         ; WEEK SEPARATOR
      (let* ((day (cadr row))
             (start-time (second-count year month day 0 0 0))
             (end-time (+ start-time seconds-in-a-day))
             (today (and today-info (>= (current-time) start-time) (<= (current-time) end-time)))
             (event-color-1 (the-event-color (first-half-day-interval year month day)  day))       ; not used in this version
             (event-color-2 (the-event-color (second-half-day-interval year month day) day)))      ; not used in this version
        (cond ((and (even? day) (not today)) (list green1 green1 green1 green1))
              ((and (even? day) today) (list red red green1 green1))
              ((and (not (even? day)) (not today)) (list green2 green2 green2 green2))
              ((and (not (even? day)) today) (list red red green2 green2))))))

    ; week separator predicate: NOT USED ANY MORE. SEPARATORS AFTER SUNDAY HOW HAS THEIR OWN ROW. THUS ALWAYS FALSE
    (lambda (row) #f)

    ; font-size:
    calendar-font-size

    (month-list-with-week-separators (make-second-counts month year) days year month)

)))

; Given the second-count-list for a month, return a list of (second-count brief-weekday day tasks-1 tasks-2) for each day i the month
; Week separators are returned as (0 "" "" "" "")
(define (month-list-with-week-separators second-count-list days year month)
  (let* ((entries  (map2 (lambda (day sec-count)
                          (list sec-count (brief-weekday sec-count) day (tasks year month day 1) (tasks year month day 2)))
                  
                        (number-interval 1 days)
                        second-count-list
                        ))
         (entries-with-sep (week-separator-extend entries)))
   (map cdr entries-with-sep)))

; Introduce week separators into entries
(define (week-separator-extend entries)
  (cond ((null? entries) '())
        ((= 7 (weekday-number (caar entries))) (cons (car entries) (cons (list 0 (week-sep) (week-sep) (week-sep) (week-sep)) (week-separator-extend (cdr entries)))))
        (else (cons (car entries) (week-separator-extend (cdr entries))))))

; enumerate years and months, starting from start-year and start-month. The entries in the list
; are of the form (y . m). The length of the list is number-of-months
(define (year-month-list start-year start-month number-of-months)
  (cond ((= 0 number-of-months) '())
        ((= start-month 12) (cons (cons start-year start-month) (year-month-list (+ 1 start-year) 1 (- number-of-months 1))))
        ((< start-month 12) (cons (cons start-year start-month) (year-month-list start-year (+ 1 start-month) (- number-of-months 1)))) 
        (else (error "year-month-list problem"))))

;;; The main calendar function. 
;;; In this section we find the most important function at all, the calendar function which generates a calendar.

;; Return a HTML calendar for year, from-months, and running through number-of-months. 
;; This is the absolute main function of the tool. 
;; As an example, (calendar 1999 2 3) will produce a calendar of February, March, and April in 1999.
;; If the optional parameter show-today is #t, the current date is shown in red.
;; .pre-condition The variable calendar-events must be defined prior to the call of this function
;; .internal-references "calendar events" "SECTION1"
(define (calendar year from-month number-of-months . show-today)
 (let ((today-info (if (null? show-today) #f (car show-today))))
  (table-2 1 (make-list number-of-months month-width)
             (make-list number-of-months calendar-background-color)
             (month-list from-month year number-of-months)
     (list 
        (map (lambda (y-m) (month-column (cdr y-m) (car y-m) today-info)) (year-month-list year from-month number-of-months))))))

; -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define (week-tasks year month day part-of-day)
 (let* ((cal-interval (cond ((= 1 part-of-day) (first-half-day-interval year month day))
                            ((= 2 part-of-day) (second-half-day-interval year month day))
                            (else (error "task half day problem"))))
        (matching-events 
          (find-all
             (lambda (e) (interval-overlap? cal-interval (event-interval e)))
             calendar-events))
        (wnc ""))
  (string-append (if (equal? "" wnc) "" (i wnc)) (boil-together matching-events ""))))

; Make a week calendar for the week that contains the time t0 (a second count).
; (define (week-calendar t0)
;   (let* ((dec-t0 (time-decode t0))
;          (wdn (weekday-number t0))
;          (day-start (time-encode (year-of-time dec-t0) (month-of-time dec-t0) (day-of-time dec-t0) 0 0 0))
;          (week-start (- day-start (* (- wdn 1) seconds-in-a-day)))
;          (decoded-week-start (time-decode week-start))
;          (week-end (- (+ (* 7 seconds-in-a-day) week-start) 1))
;          (week-start-time-list (map (lambda (i) (+ week-start (* i seconds-in-a-day))) (number-interval 0 6)))
;          (weekday-list (if (eq? language-preference 'english)
;                            (list "Monday" "Tuesday" "Wednesday" "Thursday" "Friday" "Saturday" "Sunday")
;                            (list "Mandag" "Tirsdag" "Onsdag" "Torsdag" "Fredag" "Lrdag" "Sndag")
;                        ))
;         )
;    (con
;      (text-choice "Ugestart: " "Start of week: ") (car (date-time week-start)) (br)
;      (text-choice "Ugenummer: " "Week number: ") (as-string (danish-week-number week-start)) (p)
;      
;      (table-2 1 (list 20 20 200 200)
;               (make-list 4 green2)
;               (list "" "" "" "")
;               (map2 (lambda (weekday daystart-time)
;                       (let* ((decoded-daystart-time (time-decode daystart-time))
;                              (day (day-of-time decoded-daystart-time))
;                              (month (month-of-time decoded-daystart-time))
;                              (year (year-of-time decoded-daystart-time))
;                             )
;                         (list weekday (as-string day) (week-tasks year month day 1) (week-tasks year month day 2))))
;                     weekday-list  week-start-time-list)))))

(define (week-calendar t0)
  (let* ((dec-t0 (time-decode t0))
         (wdn (weekday-number t0))
         (day-start (time-encode (year-of-time dec-t0) (month-of-time dec-t0) (day-of-time dec-t0) 0 0 0))
         (week-start (- day-start (* (- wdn 1) seconds-in-a-day)))
         (decoded-week-start (time-decode week-start))
         (week-end (- (+ (* 7 seconds-in-a-day) week-start) 1))
         (week-start-time-list (map (lambda (i) (+ week-start (* i seconds-in-a-day))) (number-interval 0 6)))
         (weekday-list (if (eq? language-preference 'english)
                           (list "Monday" "Tuesday" "Wednesday" "Thursday" "Friday" "Saturday" "Sunday")
                           (list "Mandag" "Tirsdag" "Onsdag" "Torsdag" "Fredag" "Lrdag" "Sndag")
                       ))
         (weekday-assoc-list (pair-up weekday-list week-start-time-list))
        )
   (con
     (text-choice "Ugestart: " "Start of week: ") (car (date-time week-start)) (br)
     (text-choice "Ugenummer: " "Week number: ") (as-string (danish-week-number week-start)) (p)
     
     (cal-table
       1 
       (list 20 20 200 200)
       (lambda (row)
         (let* ((weekday (car row))
                (start-time (cdr (assoc weekday weekday-assoc-list)))
                (cur-time (current-time)))
          (cond ((and (<= start-time cur-time) (<= cur-time (+ start-time seconds-in-a-day))) (list red red green2 green2 ))
                (else (list green2 green2 green2 green2)))))
       (lambda (row) #f) ; not relevant
       3
       (map2 (lambda (weekday daystart-time)
               (let* ((decoded-daystart-time (time-decode daystart-time))
                      (day (day-of-time decoded-daystart-time))
                      (month (month-of-time decoded-daystart-time))
                      (year (year-of-time decoded-daystart-time))
                     )
                 (list weekday (as-string day) (week-tasks year month day 1) (week-tasks year month day 2))))
             weekday-list  week-start-time-list)))))